#ifndef BMP__H_
#define BMP__H_

/**
  * @file BMP.h
  *
  * Library to work with Bitmaps.
  *
  * @author Marcell Dietl, Sven Eric Panitz
  * @date 25.01.2010
  *
  **/

#include <stdlib.h>
#include <stdio.h>

typedef struct {
  /** Intensity of red (0 - 255). **/
  unsigned char red;
  /** Intensity of green (0 - 255). **/
  unsigned char green;
  /** Intensity of blue (0 - 255). **/
  unsigned char blue;
} Color;

const static Color BLACK = {0,0,0};
const static Color WHITE = {255,255,255};
const static Color RED   = {255,0,0};
const static Color GREEN = {0,255,0};
const static Color BLUE  = {0,0,255};
const static Color YELLOW= {255,255,0};
const static Color PURPLE= {128,0,128};
const static Color GREY  = {128,128,128};

typedef struct {
  /** Width of the Bitmap in pixels. **/
  int width;
  /** Height of the Bitmap in pixels. **/
  int height;
  /** Color of the Bitmap. **/
  Color* imagedata ;
} Bmp;

/**
  * Allocate memory for a new Bitmap.
  *
  * The function allocates memory needed to work with
  * a Bitmap of width * height pixels.
  *
  * @param width The width of the Bitmap in pixels.
  * @param height The height of the Bitmap in pixels.
  *
  * @result Pointer to the generated Bitmap in memory.
  *
  **/
Bmp* newBmp(int width,int  height);

/**
  * Delete the Bitmap in memory.
  *
  * The function deletes the Bitmap in memory by simply
  * releasing the used addresses making a call to free().
  *
  * @param this Pointer to the Bitmap in memory.
  *
  **/
void deleteBmp(Bmp* this);

/**
  * Save the final Bitmap to a file on the disk.
  *
  * The function takes a pointer to a Bitmap in memory
  * and saves it in a specified file on the local hard
  * drive. Thus making it possible to view the results.
  *
  * @param this Pointer to the Bitmap in memory.
  * @param fileName Name to be used for the saved *.bmp file.
  *
  **/
void writeBmpToFile(Bmp* this,char* fileName);

/**
  * Set a point with a given color.
  *
  * The function sets a point of one pixel on the position
  * of x/y with the given color.
  *
  * @param b Pointer to the Bitmap in memory.
  * @param c Color the be used for the point set.
  * @param x Position on the X-axis.
  * @param y Position on the Y-axis.
  *
  **/
void setBmpPoint(Bmp* b,Color c,int x, int y);

#endif
