#include "BMP.h"
#include "DIAGRAM.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

void background(Bmp* this, Color c) {
	fillRect(this, c, 0, 0, this->width, this->height);
}

void fillRect(Bmp* this, Color c, int x, int y, int w, int h) {
	if (h >= w) {
		for (int i = 0; i <= h; i++) {
			for (int j = 0; j <= w; j++) {
				setBmpPoint(this, c, j+x, i+y); } }
	} else {
		for (int i = 0; i <= w; i++) {
			for (int j = 0; j <= h; j++) {
				setBmpPoint(this, c, i+x, j+y); } }
	}
}

/* Copied from skript.pdf *************************************************/
void swapValues(int* x,int* y){
	int z= *x;
	*x= *y;
	*y = z;
}

void drawLine(Bmp* this, Color c, int fromX, int fromY, int toX, int toY) {
	int xDiv = abs(toX-fromX);
	int yDiv = abs(toY-fromY);

	if (fromX==toX && fromY==toY){
		setBmpPoint(this,c,fromX,fromY);
	}else if (xDiv>yDiv){
		if (fromX>toX){
			swapValues(&fromY,&toY);
			swapValues(&fromX,&toX);
		}
		double steigung= ((double)toY-fromY)/(toX-fromX);
		for(int x=fromX;x<=toX;x++){
			setBmpPoint(this,c,x,fromY+(int)(x-fromX)*steigung);
		}
	}else{
		if (fromY>toY){
			swapValues(&fromY,&toY);
			swapValues(&fromX,&toX);
		}
		double steigung= ((double)toX-fromX)/(toY-fromY);
		for(int y=fromY;y<=toY;y++){
			setBmpPoint(this,c,fromX+(int)(y-fromY)*steigung,y);
		}
	}
}
/**************************************************************************/

void fillCircle(Bmp* this, Color c, int x, int y, int radius) {
	for (int i = 0; i <= radius; i++) {
		for (int j = 0; j <= radius; j++) {
			double hypo = sqrt(i*i+j*j);
			if (hypo < radius) {
				setBmpPoint(this, c, i+x, j+y);
				setBmpPoint(this, c, i+x, y-j);
				setBmpPoint(this, c, x-i, y-j);
				setBmpPoint(this, c, x-i, j+y);
			}
		}
	}
}

Bmp* barChart(double values[], int valueLength) {
	int w = valueLength * 10 * 2 + 10;
	int h = (int)values[0];
	for (int i = 0; i < valueLength; i++) {
		if ((int)values[i] > h) {
			h = (int)values[i];
		}
	}
	h += 10;

	Bmp* this = newBmp(w*5, h*5);
	background(this, WHITE);

	srand(time(NULL));
	for (int i = 0; i < valueLength; i++) {
		Color c = {rand() % 256, rand() % 256, rand() % 256};
		fillRect(this, c, (((i+1)*10)+(i*10))*5, 0*5, 10*5, (int)values[i]*5);
	}
	
	return this;
}

Bmp* pieChart(double values[], int valueLength) {
	int width  = 500;
	int height = 500;
	int radius = 200;
	int x      = width/2;
	int y      = height/2;

	Bmp* this = newBmp(width, height);
	background(this, WHITE);
	fillCircle(this, BLACK, x, y, radius);

	srand(time(NULL));
	for (int h = valueLength-1; h >= 0; h--) {
		Color c = {rand() % 256, rand() % 256, rand() % 256};

		for (int i = 0; i <= radius; i++) {
			for (int j = 0; j <= radius; j++) {
				double hypo  = sqrt(i*i+j*j);
				double alpha = (hypo!=0) ? asin(j/hypo) * 180 / 3.14159 : 0;
			
				if ((int)values[h] <= 90) {
					if (hypo < radius && alpha >= 90-values[h]) {
						setBmpPoint(this, c, j+y, i+x);
					}
				} else if ((int)values[h] > 90 && (int)values[h] <= 180) {
					hypo<radius ? setBmpPoint(this, c, i+x, j+y) : 0;
					if (hypo < radius && alpha >= 180-values[h]) {
						setBmpPoint(this, c, x-i, j+y);
					}
				} else if ((int)values[h] > 180 && (int)values[h] <= 270) {
					hypo < radius ? setBmpPoint(this, c, i+x, j+y) : 0;
					hypo < radius ? setBmpPoint(this, c, x-i, j+y) : 0;
					if (hypo < radius && alpha >= 270-values[h]) {
						setBmpPoint(this, c, y-j, x-i);
					}
				} else {
					hypo<radius ? setBmpPoint(this, c, i+x, j+y) : 0;
					hypo<radius ? setBmpPoint(this, c, x-i, j+y) : 0;
					hypo<radius ? setBmpPoint(this, c, x-i, y-j) : 0;
					if (hypo < radius && alpha >= 360-values[h]) {
						setBmpPoint(this, c, i+x, y-j);
					}
				}
			}
		}
	}

	return this;
}
